// ======================================================================
//
// Bitmap.h
// jeff grills
//
// copyright 1997 origin systems inc
//
// ======================================================================

#ifndef BITMAP_H
#define BITMAP_H

// ======================================================================

class BitmapData;
class Iff;
class Palette;
class Material;
#include "ColorVector.h"

// ======================================================================

class Bitmap
{
	// for the bitmap cut-out tool
	friend class MFCAppDoc;
	friend class MFCAppView;

private:

	struct UV
	{
		real           u, v;
	};

	struct Frame
	{
		const Material *material;
		int             width;
		int             height;
		int             hotX;
		int             hotY;
		UV              uv[4];

		Frame(void);
		~Frame(void);
		
		Frame &operator =(const Frame &newData);

		void init(void);
		void reset(void);
		void setMaterial(const Material *newMaterial);
	};

private:

	static Bitmap *firstBitmap;

	Bitmap        *nextBitmap;

	int            numFrames;
	Frame         *frameData;

private:

	void init(void);
	void createFrames(int cnt);
	void load(Iff &iff);
	void load_0001(Iff &iff);
	void load_0000(Iff &iff);

private:

	Bitmap(void);
	static void draw(int x, int y, const Frame &newFrame, const ColorVector &lighting=ColorVector::fullyLit, real a=CONST_REAL(0), real scale=CONST_REAL(1));

private:

	// disable copy constructor and assignment operator
	Bitmap(const Bitmap &);
	Bitmap &operator =(const Bitmap &);

public:

	static void debugDump(void);

public:

	explicit Bitmap(const Material *material);
	explicit Bitmap(const char *fileName);
	explicit Bitmap(Iff &iff);
	~Bitmap(void);

	void write(Iff *iff) const;
	void write(const char *filename) const;

	int  frames(void) const;

	int  width(int frame=0) const;
	int  height(int frame=0) const;
	void getHotspot(int *x, int *y, int frame=0) const;

	void draw(int x=0, int y=0, int frame=0, const ColorVector &lighting=ColorVector::fullyLit, real a=CONST_REAL(0)) const;
};

// ======================================================================

inline int Bitmap::frames(void) const
{
	return numFrames;
}

// ----------------------------------------------------------------------

inline int Bitmap::width(int frame) const
{
	return frameData[frame].width;
}

// ----------------------------------------------------------------------

inline int Bitmap::height(int frame) const
{
	return frameData[frame].height;
}

// ======================================================================

#endif
